{
Copyright (c) 2012 Yvon Massé
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}

unit Unit1;

{$MODE Delphi}

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, Math,
  {$IFDEF WIN32}
    Windows, ShellApi;
  {$ENDIF}
  {$IFDEF LINUX}
    Process;
  {$ENDIF}

type
  TForm1 = class(TForm)
    ChkPa: TCheckBox;
    ChkPb: TCheckBox;
    ChkPc: TCheckBox;
    ChkGa: TCheckBox;
    ChkGb: TCheckBox;
    ChkGC: TCheckBox;
    EdPa: TEdit;
    EdPb: TEdit;
    EdPc: TEdit;
    EdGa: TEdit;
    EdGb: TEdit;
    EdGc: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    BtCalcul: TButton;
    BtRaz: TButton;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Conf: TComboBox;
    Info: TMemo;
    BtLisez: TButton;
    BtConfig: TButton;
    procedure ChkPaClick(Sender: TObject);
    procedure ChkPbClick(Sender: TObject);
    procedure ChkPcClick(Sender: TObject);
    procedure ChkGaClick(Sender: TObject);
    procedure ChkGbClick(Sender: TObject);
    procedure ChkGcClick(Sender: TObject);
    procedure BtRazClick(Sender: TObject);
    procedure EdPaExit(Sender: TObject);
    procedure EdPbExit(Sender: TObject);
    procedure EdPcExit(Sender: TObject);
    procedure EdGaExit(Sender: TObject);
    procedure EdGbExit(Sender: TObject);
    procedure EdGcExit(Sender: TObject);
    procedure BtCalculClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ConfChange(Sender: TObject);
    procedure BtLisezClick(Sender: TObject);
    procedure BtConfigClick(Sender: TObject);
  private
    { Déclarations privées }
  public
    { Déclarations publiques }
  end;

const
dr = Pi/180; //Conversion degré/radian
prc = 0;

var
  Form1: TForm1;
  Tbol: Array[0..5] of Boolean;
  Tvar: Array[0..5, 0..1] of Real;
  Etiq: Array[0..5] of String;
  Comp: Array[0..5] of Boolean;
  Srot: Array[3..5] of Integer;
  Grot: Integer;

implementation

{$R *.lfm}

function deci(n: Real; m: Integer): Real;
// Limite le nombre de décimals de n
begin
  if m = 0 then
    Result := n
  else
    Result := Round(n*m)/m;
end;

function egal(x, y: Real): Boolean;
// Vérifie l'égalité des réels x et y
begin
  Result := (x > 0.9999999999*y) and (x < 1.0000000001*y);
end;

procedure vals(ch, sp: String; var vl: Real; out er: Boolean );
// Evalue la chaîne ch pouvant contenir des séparateurs sp pour le système sexagésimal
var
  i, e, b: Integer;
  v: Real;
begin
  vl := 0; b := 1;
  er := False;
  if Length(ch) = 0 then
    er := True
  else
    repeat
      i := Pos(sp, ch); // Recherche de sp dans ch
      if i = 0 then
        Val(ch, v, e)   // sp non trouvé, évaluation de ch
      else
      begin
        Val(Copy(ch, 1, i - 1), v, e); // Evaluation du chiffre avant sp
        if i = Length(ch) then
          i := 0            // L'évaluation est terminée
        else
          Delete(ch, 1, i); // Sinon on supprime la partie évaluée
      end;
      if e = 0 then
      begin                 // Pas d'erreur, on prend en compte 
        vl := vl + v/b;
        b := 60*b; if v < 0 then b := -b;
      end
      else
        er := True;
    until er or (i = 0);
end;

function cas1(): Integer;
// Résolution du triangle dont on connait les 3 côtés (a, b, c)
// Solution unique si le triangle est possible
var
  pa, pb, pc, Ga, Gb, Gc: Real;  // Pour les angles en radian, appellation classique : a, b, c, A, B, C
  p: Real;
begin
  pa := Tvar[0, 0]*dr; pb := Tvar[1, 0]*dr; pc := Tvar[2, 0]*dr;
  p := (pa + pb + pc)/2;  //Calcul du demi-périmètre
  if p >= Pi then
    Result := 1  // Le demi-périmètre d'un triangle sphérique ne peut pas dépasser Pi
  else
    if (pa >= p) or (pb >= p) or (pc >= p) then
      Result := 1 // Chaque coté d'un triangle ne peut pas dépasser le demi-périmètre (autre forme de l'inégalité triangulaire)
    else
    begin
      Ga := ArcCos((Cos(pa) - Cos(pb)*Cos(pc))/Sin(pb)/Sin(pc));
      Gb := ArcCos((cos(pb) - Cos(pc)*Cos(pa))/Sin(pc)/Sin(pa));
      Gc := ArcCos((Cos(pc) - Cos(pa)*Cos(pb))/Sin(pa)/Sin(pb));
      Tvar[3, 0] := Ga/dr; Tvar[3, 1] := Ga/dr;
      Tvar[4, 0] := Gb/dr; Tvar[4, 1] := Gb/dr;
      Tvar[5, 0] := Gc/dr; Tvar[5, 1] := Gc/dr;
      Result := 0;
    end;
end;

function cas3(): Integer;
// Résolution du triangle dont on connait 2 côtés et l'angle entre les côtés (a, b, C)
// Toujours une solution unique
var
  pa, pb, pc, Ga, Gb, Gc: Real;  // Pour les angles en radian, appellation classique : a, b, c, A, B, C
begin
  pa := Tvar[0, 0]*dr; pb := Tvar[1, 0]*dr; Gc := Tvar[5, 0]*dr;
  pc := ArcCos(Cos(pa)*Cos(pb) + Sin(pa)*Sin(pb)*Cos(Gc));
  Ga := ArcCos((Cos(pa) - Cos(pb)*Cos(pc))/Sin(pb)/Sin(pc));
  Gb := ArcCos((Cos(pb) - Cos(pc)*Cos(pa))/Sin(pc)/Sin(pa));
  Tvar[2, 0] := pc/dr; Tvar[2, 1] := pc/dr;
  Tvar[3, 0] := Ga/dr; Tvar[3, 1] := Ga/dr;
  Tvar[4, 0] := Gb/dr; Tvar[4, 1] := Gb/dr;
  Result := 0;
end;

function cas5(): Integer;
// Résolution du triangle dont on connait 2 côtés et l'angle à l'extrémité d'un côté (a, b, A)
// Repose sur la recherche de c dans l'équation trigonométrique E : cos a = cos b.cos c + sin b.sin c.cos A
// Nombre de solution : 0, 1, 2 ou une infinité
var
  pa, pb, pc, Ga, Gb, Gc: Real;  // Pour les angles en radian, appellation classique : a, b, c, A, B, C
  pc2, Gb2, Gc2: Real;           // Pour la seconde solution
  th: Real;
begin
  pa := Tvar[0, 0]*dr; pb := Tvar[1, 0]*dr; Ga := Tvar[3, 0]*dr;
  if egal(pb, Pi/2) and egal(Ga, Pi/2) then
    if not egal(pa, Pi/2) then
      Result := 1 // L'angle pa doit être de Pi/2
    else
    begin         // E devient : 0 = 0, c peut être quelconque. C dépend de c (et vice-versa)
      pc := 3*Pi/2; Gb := Pi/2; Gc := 3*Pi/2;  // 3*Pi/2 est le codage de l'interdépendance
      Tvar[2, 0] := pc/dr; Tvar[2, 1] := pc/dr;
      Tvar[4, 0] := Gb/dr; Tvar[4, 1] := Gb/dr;
      Tvar[5, 0] := Gc/dr; Tvar[5, 1] := Gc/dr;
      Result := 0;
    end
  else
  begin
    th := Cos(pa)/Sqrt(Cos(pb)*Cos(pb) + Sin(pb)*Sin(pb)*Cos(Ga)*Cos(Ga));
    if Abs(th) > 1 then
      Result := 1 // Pas de solution
    else
    begin
      pc := Arctan2(Sin(pb)*Cos(Ga), Cos(pb)) + Arccos(th); if (pc > Pi) or egal(pc, Pi) then pc := pc - 2*Pi;
      pc2 := Arctan2(Sin(pb)*Cos(Ga), Cos(pb)) - ArcCos(th); if (pc2 < -Pi) and not egal(pc2, -Pi) then pc2 := pc2 + 2*Pi;
      if pc < pc2 then  // Organisation des résultats en valeurs décroissantes
      begin
        th := pc; pc := pc2; pc2 := th;
      end;
      if (pc <= 0) and (pc2 <= 0) then
        Result := 1 // Pas de solution, les résultats ne sont pas dans l'interval ]0, Pi[
      else
      begin
        if pc2 <= 0 then pc2 := pc; // pc2 n'est pas acceptable, une seule solution
        Gb := ArcCos((Cos(pb) - Cos(pc)*Cos(pa))/Sin(pc)/Sin(pa));
        Gc := ArcCos((Cos(pc) - Cos(pa)*Cos(pb))/Sin(pa)/Sin(pb));
        Gb2 := ArcCos((Cos(pb) - Cos(pc2)*Cos(pa))/Sin(pc2)/Sin(pa));
        Gc2 := ArcCos((Cos(pc2) - Cos(pa)*Cos(pb))/Sin(pa)/Sin(pb));
        Tvar[2, 0] := pc/dr; Tvar[2, 1] := pc2/dr;
        Tvar[4, 0] := Gb/dr; Tvar[4, 1] := Gb2/dr;
        Tvar[5, 0] := Gc/dr; Tvar[5, 1] := Gc2/dr;
        Result := 0;
      end;
    end;
  end;
end;  

procedure invi();
// Echange des paramètres 0/1 et 3/4
var
  i: Integer;
  m0, m3: Real;
begin
  for i := 0 to 1 do
  begin
    m0 := Tvar[0, i]; m3 := Tvar[3, i];
    Tvar[0, i] := Tvar[1, i]; Tvar[3, i] := Tvar[4, i];
    Tvar[1, i] := m0; Tvar[4, i] := m3;
  end;
end;

procedure invj();
// Echange des paramètres 1/2 et 4/5
var
  i: Integer;
  m1, m4: Real;
begin
  for i := 0 to 1 do
  begin
    m1 := Tvar[1, i]; m4 := Tvar[4, i];
    Tvar[1, i] := Tvar[2, i]; Tvar[4, i] := Tvar[5, i];
    Tvar[2, i] := m1; Tvar[5, i] := m4;
  end;
end;

procedure pol();
// Passage au triangle polaire
var
  i, j: Integer;
  m: Real;
begin
  // Angles supplémentaire
  for j := 0 to 1 do
    for i := 0 to 5 do
      Tvar[i, j] := 180 - Tvar[i, j];
  // Inversion des côtés et des angles
  for j := 0 to 1 do
    for i := 0 to 2 do
    begin
      m := Tvar[i, j]; Tvar[i,j] := Tvar[i + 3, j]; Tvar[i + 3, j] := m;
    end;
end;

procedure TForm1.BtCalculClick(Sender: TObject);
var
  chx: Word;
  i, p2, nbpar, conf, cont, sol: Integer;
  my: Real;
  merr, seq, ch1, ch2: String;
  su: Boolean;
begin
  //Pour quitter un éventuel champ de saisie lors de la frappe de la touche 'Entrée'
  if ActiveControl = EdPa then EdPaExit(nil);
  if ActiveControl = EdPb then EdPbExit(nil);
  if ActiveControl = EdPc then EdPcExit(nil);
  if ActiveControl = EdGa then EdGaExit(nil);
  if ActiveControl = EdGb then EdGbExit(nil);
  if ActiveControl = EdGc then EdGcExit(nil);
  p2 := 1;
  nbpar := 0;
  conf := 0;
  for i := 0 to 5 do  // Calcul de la configuration
  begin
    if Tbol[i] then
    begin
      Inc(nbpar);
      conf := conf + p2;
    end;
    p2 := 2*p2;
  end;
  if (nbpar < 3) or (nbpar > 3) then  // Vérification du nombre d'angles saisis
  begin
    if nbpar < 3 then merr := 'Il faut 3 paramètres';
    if nbpar > 3 then merr := 'Seulement 3 paramètres';
    MessageDlgPos(merr, mtError, [mbOK], 0, Form1.Left, Form1.Top);
  end
  else
  begin
    // Décoche la case s'il n'y a pas de valeur saisie
    if ChkPa.Checked and not Tbol[0] then ChkPa.Checked := False;
    if ChkPb.Checked and not Tbol[1] then ChkPb.Checked := False;
    if ChkPc.Checked and not Tbol[2] then ChkPc.Checked := False;
    if ChkGa.Checked and not Tbol[3] then ChkGa.Checked := False;
    if ChkGb.Checked and not Tbol[4] then ChkGb.Checked := False;
    if ChkGc.Checked and not Tbol[5] then ChkGc.Checked := False;
    case conf of    // Acquisition de la séquence de traitement correspondant à la configuration
    7: seq := '1';
    11: seq := '5';
    19: seq := 'i5i';
    35: seq := '3';
    13: seq := 'j5j';
    21: seq := 'j3j';
    37: seq := 'ji5ij';
    14: seq := 'ij3ji';
    22: seq := 'ij5ji';
    38: seq := 'iji5iji';
    56: seq := 'p1p';
    25: seq := 'p5p';
    26: seq := 'pi5ip';
    28: seq := 'p3p';
    41: seq := 'pj5jp';
    42: seq := 'pj3jp';
    44: seq := 'pji5ijp';
    49: seq := 'pij3jip';
    50: seq := 'pij5jip';
    52: seq := 'piji5ijip';
    end;
    cont := 0;
    for i := 1 to Length(seq) do  // Exécution de la séquence de traitement
      case seq[i] of
      'i': invi();
      'j': invj();
      'p': begin
        pol();
        if cont > 1 then cont := cont + 10; // Pour future évolution vers un codage du message d'erreur
      end;
      '1': cont := cas1();
      '3': cont := cas3();
      '5': cont := cas5();
      end;
    if cont = 0 then
    begin                          // Pas d'erreur, affichage des résultats
      if (conf = 7) and (Grot <> 0) then //Choix du signe des angles
      begin
        chx := MessageDlgPos(Etiq[3] + ' positif ?', mtConfirmation, [mbYes, mbNo], 0, Form1.Left, Form1.Top);
        Grot := Srot[3]; if (chx = mrNo) then Grot := -Grot;
      end;
      su := True;   // A priori, solution unique
      sol := 0;     // Dans ce cas, utilisation de l'indice 0
      for i := 0 to 5 do
      begin
        if conf mod 2 = 0 then
        begin               // i correspond à l'indice d'un résultat
          if (Tvar[i, 0] <> Tvar[i, 1]) and su then // Choix de la solution s'il y en a 2
          begin
            su := False;
            my := deci((Tvar[i, 0] + Tvar[i, 1])/2, 100);
            if Comp[i] then my := 90 - my;     // La question est toujours posée pour un côté
            chx := MessageDlgPos('Deux solutions : ' + Etiq[i] + ' > ' + FloatToStr(my) + ' ?', mtConfirmation, [mbYes, mbNo], 0, Form1.Left, Form1.Top);
            if (chx = mrNo) xor (Tvar[i, 0] < Tvar[i, 1]) xor Comp[i] then sol := 1;
          end;
          if Sin(Tvar[i, sol]*dr) > -0.5 then  // Affichage des résultats "normaux"
          begin
            my := deci(Tvar[i, sol], prc);
            if Comp[i] then                    // Complémentation ?
              if i < 3 then
                my := 90 - my
              else
                my := 180 - my;
            if (i >= 3) and (Grot <> 0) then   // Signe de l'angle
              my := Grot*Srot[i]*my;
            case i of
            0: EdPa.Text := FloatToStr(my);
            1: EdPb.Text := FloatToStr(my);
            2: EdPc.Text := FloatToStr(my);
            3: EdGa.Text := FloatToStr(my);
            4: EdGb.Text := FloatToStr(my);
            5: EdGc.Text := FloatToStr(my);
            end;
          end
          else                                 // Affichage des résultats symboliques
          begin
            ch1 := '= '; ch2 := '';
            if Grot*Srot[(i mod 3) + 3] = -1 then
              if Comp[(i mod 3) + 3] then
                if Comp[i mod 3] then
                    ch1 := '= -90 - '
                else
                  if i < 3 then
                    ch2 := ' + 180'
                  else
                    ch2 := ' - 180'
              else
                if Comp[i mod 3] then
                  if i < 3 then
                    ch2 := ' + 90'
                  else
                    ch2 := ' - 90'
                else
                  ch1 := '= -'
            else
              if Comp[(i mod 3) + 3] then
                if Comp[i mod 3] then
                  if i < 3 then
                    ch2 := ' - 90'
                  else
                    ch2 := ' + 90'
                else
                  ch1 := '= 180 - '
              else
                if Comp[i mod 3] then
                  ch1 := '= 90 - ';
            case i of
            0: EdPa.Text := ch1 + ChkGa.Caption + ch2;
            1: EdPb.Text := ch1 + ChkGb.Caption + ch2;
            2: EdPc.Text := ch1 + ChkGc.Caption + ch2;
            3: EdGa.Text := ch1 + ChkPa.Caption + ch2;
            4: EdGb.Text := ch1 + ChkPb.Caption + ch2;
            5: EdGc.Text := ch1 + ChkPc.Caption + ch2;
            end;
          end;
        end;
        conf := conf div 2;
      end;
    end
    else
      MessageDlgPos('Pas de solution', mtError, [mbOK], 0, Form1.Left, Form1.Top);
  end;
end;

procedure TForm1.ChkPaClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkPa.Checked = True then
  begin
    EdPa.TabStop := True;
    EdPa.ReadOnly := False;
    EdPa.Color := clWindow;
    EdPaExit(nil)
  end
  else
  begin
    EdPa.TabStop := False;
    EdPa.ReadOnly := True;
    EdPa.Color := clForm;
    Tbol[0] := False;
  end;
end;

procedure TForm1.ChkPbClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkPb.Checked = True then
  begin
    EdPb.TabStop := True;
    EdPb.ReadOnly := False;
    EdPb.Color := clWindow;
    EdPbExit(nil)
  end
  else
  begin
    EdPb.TabStop := False;
    EdPb.ReadOnly := True;
    EdPb.Color := clForm;
    Tbol[1] := False;
  end;
end;

procedure TForm1.ChkPcClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkPc.Checked = True then
  begin
    EdPc.TabStop := True;
    EdPc.ReadOnly := False;
    EdPc.Color := clWindow;
    EdPcExit(nil)
  end
  else
  begin
    EdPc.TabStop := False;
    EdPc.ReadOnly := True;
    EdPc.Color := clForm;
    Tbol[2] := False;
  end;
end;

procedure TForm1.ChkGaClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkGa.Checked = True then
  begin
    EdGa.TabStop := True;
    EdGa.ReadOnly := False;
    EdGa.Color := clWindow;
    EdGaExit(nil)
  end
  else
  begin
    EdGa.TabStop := False;
    EdGa.ReadOnly := True;
    EdGa.Color := clForm;
    Tbol[3] := False;
  end;
end;

procedure TForm1.ChkGbClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkGb.Checked = True then
  begin
    EdGb.TabStop := True;
    EdGb.ReadOnly := False;
    EdGb.Color := clWindow;
    EdGbExit(nil)
  end
  else
  begin
    EdGb.TabStop := False;
    EdGb.ReadOnly := True;
    EdGb.Color := clForm;
    Tbol[4] := False;
  end;
end;

procedure TForm1.ChkGcClick(Sender: TObject);
// Gestion des propriétés du champ de saisie correspondant
begin
  if ChkGc.Checked = True then
  begin
    EdGc.TabStop := True;
    EdGc.ReadOnly := False;
    EdGc.Color := clWindow;
    EdGcExit(nil)
  end
  else
  begin
    EdGc.TabStop := False;
    EdGc.ReadOnly := True;
    EdGc.Color := clForm;
    Tbol[5] := False;
  end;
end;

procedure TForm1.BtRazClick(Sender: TObject);
begin
  ChkPa.Checked := False; EdPa.Text := '';
  ChkPb.Checked := False; EdPb.Text := '';
  ChkPc.Checked := False; EdPc.Text := '';
  ChkGa.Checked := False; EdGa.Text := '';
  ChkGb.Checked := False; EdGb.Text := '';
  ChkGc.Checked := False; EdGc.Text := '';
end;

procedure TForm1.EdPaExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkPa.Checked then
  begin
    vals(EdPa.Text, '°', Tvar[0, 0], err);
    if err then
      EdPa.Text := ''
    else
    begin
      if Comp[0] then Tvar[0, 0] := 90 - Tvar[0, 0];
      if (Tvar[0, 0] <= 0) or (Tvar[0, 0] >= 180) then EdPa.Text := '';
    end;
    Tbol[0] := EdPa.Text <> '';
  end;
end;

procedure TForm1.EdPbExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkPb.Checked then
  begin
    vals(EdPb.Text, '°', Tvar[1, 0], err);
    if err then
      EdPb.Text := ''
    else
    begin
      if Comp[1] then Tvar[1, 0] := 90 - Tvar[1, 0];
      if (Tvar[1, 0] <= 0) or (Tvar[1, 0] >= 180) then EdPb.Text := '';
    end;
    Tbol[1] := EdPb.Text <> '';
  end;
end;

procedure TForm1.EdPcExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkPc.Checked then
  begin
    vals(EdPc.Text, '°', Tvar[2, 0], err);
    if err then
      EdPc.Text := ''
    else
    begin
      if Comp[2] then Tvar[2, 0] := 90 - Tvar[2, 0];
      if (Tvar[2, 0] <= 0) or (Tvar[2, 0] >= 180) then EdPc.Text := '';
    end;
    Tbol[2] := EdPc.Text <> '';
  end;
end;

procedure TForm1.EdGaExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkGa.Checked then
  begin
    vals(EdGa.Text, '°', Tvar[3, 0], err);
    if err then
      EdGa.Text := ''
    else
    begin
      if Grot <> 0 then
        if Tbol[4] or Tbol[5] then
        begin
          Tvar[3, 0] := Grot*Srot[3]*Tvar[3, 0];
          if Tvar[3, 0] < 0 then
            MessageDlgPos('Incohérence de signe', mtError, [mbOK], 0, Form1.Left, Form1.Top);
        end
        else
        begin
          Tvar[3, 0] := Srot[3]*Tvar[3, 0];
          Grot := 1;
          if Tvar[3, 0] < 0 then
          begin
            Grot := -Grot;
            Tvar[3, 0] := -Tvar[3, 0];
          end;
        end;
      if Comp[3] then Tvar[3, 0] := 180 - Tvar[3, 0];
      if (Tvar[3, 0] <= 0) or (Tvar[3, 0] >= 180) then EdGa.Text := '';
    end;
    Tbol[3] := EdGa.Text <> '';
  end;
end;

procedure TForm1.EdGbExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkGb.Checked then
  begin
    vals(EdGb.Text, '°', Tvar[4, 0], err);
    if err then
      EdGb.Text := ''
    else
    begin
      if Grot <> 0 then
        if Tbol[3] or Tbol[5] then
        begin
          Tvar[4, 0] := Grot*Srot[4]*Tvar[4, 0];
          if Tvar[4, 0] < 0 then
            MessageDlgPos('Incohérence de signe', mtError, [mbOK], 0, Form1.Left, Form1.Top);
        end
        else
        begin
          Tvar[4, 0] := Srot[4]*Tvar[4, 0];
          Grot := 1;
          if Tvar[4, 0] < 0 then
          begin
            Grot := -Grot;
            Tvar[4, 0] := -Tvar[4, 0];
          end;
        end;
      if Comp[4] then Tvar[4, 0] := 180 - Tvar[4, 0];
      if (Tvar[4, 0] <= 0) or (Tvar[4, 0] >= 180) then EdGb.Text := '';
    end;
    Tbol[4] := EdGb.Text <> '';
  end;
end;

procedure TForm1.EdGcExit(Sender: TObject);
var
  err: Boolean;
begin
  if ChkGc.Checked then
  begin
    vals(EdGc.Text, '°', Tvar[5, 0], err);
    if err then
      EdGc.Text := ''
    else
    begin
      if Grot <> 0 then
        if Tbol[3] or Tbol[4] then
        begin
          Tvar[5, 0] := Grot*Srot[5]*Tvar[5, 0];
          if Tvar[5, 0] < 0 then
            MessageDlgPos('Incohérence de signe', mtError, [mbOK], 0, Form1.Left, Form1.Top);
        end
        else
        begin
          Tvar[5, 0] := Srot[5]*Tvar[5, 0];
          Grot := 1;
          if Tvar[5, 0] < 0 then
          begin
            Grot := -Grot;
            Tvar[5, 0] := -Tvar[5, 0];
          end;
        end;
      if Comp[5] then Tvar[5, 0] := 180 - Tvar[5, 0];
      if (Tvar[5, 0] <= 0) or (Tvar[5, 0] >= 180) then EdGc.Text := '';
    end;
    Tbol[5] := EdGc.Text <> '';
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  fic: TextFile;
  lin: String;
begin
  DefaultFormatSettings.DecimalSeparator := '.';
  if FileExists(GetCurrentDir + DirectorySeparator + 'TriSph.cnf') then
  begin
    AssignFile(fic, GetCurrentDir + DirectorySeparator + 'TriSph.cnf');
    Reset(fic);
    while not Eof(fic) do
    begin
      Readln(fic, lin);
      Conf.Items.Add(lin);
      while (lin <> '---') and not Eof(fic) do
        Readln(fic, lin);
    end;
    CloseFile(fic);
  end;
  Conf.ItemIndex := 0;
  ConfChange(nil);
end;

procedure TForm1.ConfChange(Sender: TObject);
// Changement 
var
  fic: TextFile;
  lin, mot: String;
  i,j: Integer;
begin
  // Remise à zéro de toute la fenêtre du programme
  BtRazClick(nil);
  for i := 0 to 5 do
  begin
    Tbol[i] := False;
    Etiq[i] := '';
    Comp[i] := False;
    if i >= 3 then Srot[i] := 0;
  end;
  Info.Clear;
  if Conf.ItemIndex = 0 then
  begin                  // Cas du triangle sphérique de base
    Etiq[0] := 'a';
    Etiq[1] := 'b';
    Etiq[2] := 'c';
    Etiq[3] := 'A';
    Etiq[4] := 'B';
    Etiq[5] := 'C';
    Grot := 0;
    Info.Text := 'Les minuscules sont les côtés du triangle sphérique'#13#10#13#10;
    Info.Text := Info.Text + 'Les majuscules correspondantes sont les angles opposés'#13#10;
  end
  else
  begin                  // Autres cas à lire dans TriSph.cnf
    AssignFile(fic, GetCurrentDir + DirectorySeparator + 'TriSph.cnf');
    Reset(fic);
    while not Eof(fic) do // Recherche du paragraphe correspondant à la sélection
    begin
      Readln(fic, lin);
      if lin = Conf.Items[Conf.ItemIndex] then Break;
      while (lin <> '---') and not Eof(fic) do
        Readln(fic, lin);
    end;
    if Eof(fic) then
      MessageDlgPos('Ce paragraphe ne se trouve plus'#13#10'dans le fichier de configuration.'#13#10'Relancer TriSph.',
                     mtError, [mbOK], 0, Form1.Left, Form1.Top)
    else
    begin
      mot := ''; j := 0;
      Readln(fic,lin); // Lecture de l'étiquette des angles
      for i := 1 to Length(lin) + 1 do
        if (i = Length(lin) + 1) or (lin[i] = ',') then
        begin
          if j < 6 then Etiq[j] := mot;
          mot := ''; Inc(j);
        end
        else
          mot := mot + lin[i];
      Readln(fic,lin); // Lecture des booléens de complément
      for i := 1 to Length(lin) do
        if (i < 7) and (lin[i] = 'O') then Comp[i - 1] := True;
      Grot := 1;
      Readln(fic,lin); // Lecture des informations de signe
      for i := 1 to Length(lin) do
        if (i < 4) then
          case lin[i] of
          'R': Srot[i + 2] := 1;
          'S': Srot[i + 2] := -1;
          else Grot := 0;
          end;
      Readln(fic,lin); // Lecture des indications
      while (lin <> '---') and not Eof(fic) do
      begin
        if lin[1] <> ';' then Info.Text := Info.Text + lin + #13#10;
        Readln(fic, lin);
      end;
      if Eof(fic) then Info.Text := Info.Text + lin + #13#10;
    end;
    CloseFile(fic);
  end;
  ChkPa.caption := Etiq[0];
  ChkPb.caption := Etiq[1];
  ChkPc.caption := Etiq[2];
  ChkGa.caption := Etiq[3];
  ChkGb.caption := Etiq[4];
  ChkGc.caption := Etiq[5];
end;

procedure TForm1.BtLisezClick(Sender: TObject);
var
  {$IFDEF WIN32}
    rt: Cardinal;
  {$ENDIF}
  {$IFDEF LINUX}
    Proc: Tprocess;
  {$ENDIF}
begin
  {$IFDEF WIN32}
    rt := ShellExecute(0, 'OPEN', Pchar('Lisezmoi.pdf'), nil, Pchar(ExtractFilePath(ParamStr(0))), SW_SHOW);
    if rt <= 32 then
    begin
      if rt = SE_ERR_FNF then
        MessageDlgPos('Le fichier Lisezmoi.pdf est absent', mtError, [mbOK], 0, Form1.Left, Form1.Top)
      else
        if rt = SE_ERR_NOASSOC then
          MessageDlgPos('Aucune application ne peut lire le fichier Lisezmoi.pdf', mtError, [mbOK], 0, Form1.Left, Form1.Top)
        else
          MessageDlgPos('Erreur à l''ouverture du fichier Lisezmoi.pdf', mtError, [mbOK], 0, Form1.Left, Form1.Top);
    end;
  {$ENDIF}
  {$IFDEF LINUX}
    if FileExists(ExtractFilePath(ParamStr(0)) + 'Lisezmoi.pdf') then
    begin
      Proc := Tprocess.Create(nil);
      Proc.CommandLine := 'evince ' + ExtractFilePath(ParamStr(0)) + 'Lisezmoi.pdf';
      try
        Proc.Execute;
      except
        begin
          MessageDlgPos('Erreur à la tentative de visualiser le fichier Lisezmoi.pdf', mtError, [mbOK], 0, Form1.Left, Form1.Top);
          Proc.free;
        end;
      end;
    end
    else
      MessageDlgPos('Le fichier Lisezmoi.pdf est absent', mtError, [mbOK], 0, Form1.Left, Form1.Top);
  {$ENDIF}
end;

procedure TForm1.BtConfigClick(Sender: TObject);
var
  {$IFDEF WIN32}
    rt: Cardinal;
  {$ENDIF}
  {$IFDEF LINUX}
    Proc: Tprocess;
  {$ENDIF}
begin
  {$IFDEF WIN32}
    rt := ShellExecute(0, 'OPEN', Pchar('Notepad'), Pchar('TriSph.cnf'), Pchar(GetCurrentDir), SW_SHOW);
    if rt <= 32 then
    begin
      if rt = SE_ERR_FNF then
        MessageDlgPos('L''application Bloc-notes est absente', mtError, [mbOK], 0, Form1.Left, Form1.Top)
      else
        MessageDlgPos('Erreur à l''ouverture du fichier TriSph.cnf', mtError, [mbOK], 0, Form1.Left, Form1.Top);
    end;
  {$ENDIF}
  {$IFDEF LINUX}
    Proc := Tprocess.Create(nil);
    Proc.CommandLine := 'gedit ' + GetCurrentDir + DirectorySeparator + 'TriSph.cnf';
    try
      Proc.Execute;
    except
      begin
        MessageDlgPos('Erreur à la tentative d''ouvrir le fichier TriSph.cnf', mtError, [mbOK], 0, Form1.Left, Form1.Top);
        Proc.free;
      end;
    end;
  {$ENDIF}
end;

end.
